# frozen_string_literal: true

module TCellAgent
  class ConfigInitializer < Configuration
    # Common config shared across agents
    attr_accessor :app_id, :api_key, :tcell_api_url,
                  :tcell_input_url, :log_dir, :fetch_policies_from_tcell,
                  :preload_policy_filename, :reverse_proxy,
                  :reverse_proxy_ip_address_header, :host_identifier,
                  :hmac_key, :password_hmac_key,
                  :js_agent_url, :js_agent_api_base_url,
                  :max_csp_header_bytes, :allow_payloads,
                  :proxy_url, :proxy_username, :proxy_password

    attr_reader :logging_options

    # Ruby only config
    attr_accessor :disable_all, :enabled, :enable_event_manager,
                  :enable_policy_polling,
                  :data_exposure

    attr_reader :instrument_for_events,
                :enable_instrumentation

    # New config
    attr_accessor :disabled_instrumentation, :instrument

    def initialize
      # ruby agent defaults
      @logging_options = {}
    end

    def logging_options=(hash)
      @logging_options = hash.each_with_object({}) do |(key, val), memo|
        memo[key.to_sym] = val
      end
    end

    # legacy config mapping
    def enabled_instrumentations=(hash)
      @disabled_instrumentation ||= []
      hash.each do |key, val|
        @disabled_instrumentation << key.to_s.strip.downcase unless TCellAgent.configuration.to_bool(val)
      end
    end

    def agent_log_dir=(path)
      @log_dir = path
    end

    def instrument_for_events=(bool)
      @instrument = bool
    end

    def enable_instrumentation=(bool)
      @instrument = bool
    end

    def enable_intercept_requests=(bool)
      @disabled_instrumentation << 'intercept_requests' unless TCellAgent.configuration.to_bool(bool)
    end
  end
end
