require 'tcell_agent/agent'

module TCellAgent
  module Hooks
    module LoginFraud
      # NOTE: mock out in tests
      def self.get_logger
        unless defined?(@login_fraud_logger)
          @login_fraud_logger = TCellAgent::ModuleLogger.new(TCellAgent.logger, name)
        end

        @login_fraud_logger
      end

      def self.report_login_event(status,
                                  env_or_header_keys,
                                  tcell_data,
                                  user_id,
                                  password,
                                  user_valid)
        return unless TCellAgent.configuration.should_intercept_requests? && tcell_data

        login_policy = TCellAgent.policy(TCellAgent::PolicyTypes::LOGINFRAUD)

        if ![TCellAgent::Hooks::V1::Login::LOGIN_FAILURE,
             TCellAgent::Hooks::V1::Login::LOGIN_SUCCESS].include?(status)
          get_logger.error("Unkown login status: #{status}")
        elsif status == TCellAgent::Hooks::V1::Login::LOGIN_FAILURE
          login_policy.report_login_failure(
            user_id,
            password,
            env_or_header_keys,
            user_valid,
            tcell_data
          )
        elsif status == TCellAgent::Hooks::V1::Login::LOGIN_SUCCESS
          login_policy.report_login_success(
            user_id,
            env_or_header_keys,
            tcell_data
          )
        end
      end
    end
  end
end

if defined?(TCellAgent::Hooks::V1::Frameworks::Rails::Login)
  TCellAgent::Hooks::V1::Frameworks::Rails::Login.module_eval do
    class << self
      alias_method :tcell_register_login_event, :register_login_event
      def register_login_event(status,
                               rails_request,
                               user_id,
                               user_valid = nil,
                               password = nil)
        TCellAgent::Instrumentation.safe_block('Rails Auth Hooks') do
          tcell_data = rails_request.env[TCellAgent::Instrumentation::TCELL_ID]
          TCellAgent::Hooks::LoginFraud.report_login_event(
            status,
            rails_request.env,
            tcell_data,
            user_id,
            password,
            user_valid
          )
        end
      end
    end
  end
end

if defined?(TCellAgent::Hooks::V1::Login)
  TCellAgent::Hooks::V1::Login.module_eval do
    class << self
      alias_method :tcell_register_login_event, :register_login_event
      def register_login_event(status,
                               session_id,
                               user_agent,
                               referrer,
                               remote_address,
                               header_keys,
                               user_id,
                               document_uri,
                               user_valid = nil,
                               password = nil)
        TCellAgent::Instrumentation.safe_block('Login Auth Hooks') do
          tcell_data = TCellAgent::Instrumentation::TCellData.new
          tcell_data.user_agent = user_agent
          tcell_data.referrer = referrer
          tcell_data.remote_address = remote_address
          tcell_data.path = document_uri
          tcell_data.session_id = session_id

          TCellAgent::Hooks::LoginFraud.report_login_event(status,
                                                           header_keys,
                                                           tcell_data,
                                                           user_id,
                                                           password,
                                                           user_valid)
        end
      end
    end
  end
end
