# See the file "LICENSE" for the full license governing this code.

require 'tcell_agent/policies/policy_types'

require 'tcell_agent/policies/appfirewall_policy'
require 'tcell_agent/policies/command_injection_policy'
require 'tcell_agent/policies/dataloss_policy'
require 'tcell_agent/policies/headers_policy'
require 'tcell_agent/policies/http_redirect_policy'
require 'tcell_agent/policies/js_agent_policy'
require 'tcell_agent/policies/login_policy'
require 'tcell_agent/policies/patches_policy'
require 'tcell_agent/policies/local_file_access'
require 'tcell_agent/policies/system_enablements'

RUST_POLICY_CLASSES = [
  TCellAgent::Policies::AppfirewallPolicy,
  TCellAgent::Policies::CommandInjectionPolicy,
  TCellAgent::Policies::HeadersPolicy,
  TCellAgent::Policies::HttpRedirectPolicy,
  TCellAgent::Policies::JsAgentPolicy,
  TCellAgent::Policies::LoginPolicy,
  TCellAgent::Policies::PatchesPolicy,
  TCellAgent::Policies::LocalFileInclusion,
  TCellAgent::Policies::SystemEnablements
].freeze

module TCellAgent
  class PoliciesManager
    attr_accessor :policies

    def initialize(native_agent)
      @native_agent = native_agent
      @policies = {}

      enablements = {}
      RUST_POLICY_CLASSES.each do |policy_class|
        @policies[policy_class.api_identifier] = policy_class.new(
          @native_agent, enablements
        )
      end

      set_dataloss_policy({ 'dlp' => {} })
    end

    def set_dataloss_policy(policies_json)
      TCellAgent::Instrumentation.safe_block('Setting DLP policy') do
        dlp_api_identifier = TCellAgent::Policies::DataLossPolicy.api_identifier
        return unless policies_json.key?(dlp_api_identifier)

        @policies[dlp_api_identifier] = TCellAgent::Policies::DataLossPolicy.new(
          policies_json[dlp_api_identifier]
        )
      end
    end

    def process_policy_json(enablements, policies_json)
      return if enablements.nil? || enablements == {}

      RUST_POLICY_CLASSES.each do |policy_class|
        @policies[policy_class.api_identifier] = policy_class.new(
          @native_agent, enablements
        )
      end

      set_dataloss_policy(policies_json)
    end
  end
end
