# frozen_string_literal: true

require 'tcell_agent/configuration'
require 'tcell_agent/instrumentation'

module TCellAgent
  require 'tcell_agent/agent'

  Authlogic::Session::Base.class_eval do
    alias_method :tcell_save, :save
    def save(&block)
      return tcell_save(&block) unless TCellAgent.configuration.should_intercept_requests?

      user_logged_in_before = !user.nil?
      success = tcell_save(&block)
      user_logged_in_after = !user.nil?

      TCellAgent::Instrumentation.safe_block('Authlogic login info') do
        user_id = nil
        password = nil
        user_valid = nil
        TCellAgent::Instrumentation.safe_block('getting userid for login form') do
          user_id = send(self.class.login_field.to_sym)
        end

        request = Authlogic::Session::Base.controller.request
        tcell_data = request.env[TCellAgent::Instrumentation::TCELL_ID]

        return success unless tcell_data

        login_policy = TCellAgent.policy(TCellAgent::PolicyTypes::LOGINFRAUD)
        if user_logged_in_before && user_logged_in_after
        # password changed or logged in as another user
        elsif !user_logged_in_before && !user_logged_in_after
          TCellAgent::Instrumentation.safe_block('checking if user is valid') do
            error_messages = errors.messages[login_field]

            user_valid = error_messages.empty?
          end

          login_policy.report_login_failure(
            user_id,
            password,
            request.env,
            user_valid,
            tcell_data
          )
        elsif !user_logged_in_before && user_logged_in_after
          tcell_data.user_id = user_id if user_id && tcell_data.user_id.nil?
          login_policy.report_login_success(
            user_id,
            request.env,
            tcell_data
          )
        end
      end

      success
    end
  end
end
