require 'tcell_agent/agent'

module TCellAgent
  module DoorkeeperInstrumentation
    Doorkeeper::TokensController.class_eval do
      alias_method :tcell_authorize_response, :authorize_response
      def authorize_response
        result = tcell_authorize_response

        TCellAgent::Instrumentation.safe_block('Doorkeeper Token Authorize') do
          return result unless TCellAgent.configuration.should_intercept_requests?

          login_policy = TCellAgent.policy(TCellAgent::PolicyTypes::LOGINFRAUD)
          tcell_data = request.env[TCellAgent::Instrumentation::TCELL_ID]

          return unless tcell_data

          headers = request.env

          if result.is_a?(Doorkeeper::OAuth::TokenResponse)
            user_id = result.token.resource_owner_id
            login_policy.report_login_success(
              user_id,
              headers,
              tcell_data
            )
          elsif result.is_a?(Doorkeeper::OAuth::ErrorResponse)
            user_id = request.POST['client_id']
            password = nil
            user_valid = nil
            login_policy.report_login_failure(
              user_id,
              password,
              headers,
              user_valid,
              tcell_data
            )
          end
        end

        result
      end
    end

    module TCellAuthorizationsNew
      def new
        super if defined?(super)

        TCellAgent::Instrumentation.safe_block('Doorkeeper Token Authorize') do
          return unless TCellAgent.configuration.should_intercept_requests?
          return unless pre_auth.error

          login_policy = TCellAgent.policy(TCellAgent::PolicyTypes::LOGINFRAUD)
          tcell_data = request.env[TCellAgent::Instrumentation::TCELL_ID]

          return unless tcell_data

          user_id = current_resource_owner.id
          password = nil
          headers = request.env
          user_valid = nil
          login_policy.report_login_failure(
            user_id,
            password,
            headers,
            user_valid,
            tcell_data
          )
        end
      end
    end

    # prepend is ruby 2+ feature
    Doorkeeper::AuthorizationsController.send(:prepend, TCellAuthorizationsNew)
  end
end
