module TCellAgent
  module DLP
    def self.handle_request_dlp_parameters(request)
      TCellAgent::Instrumentation.safe_block('Handling Dataexposure (request forms)') do
        _handle_dataexpsure_forms(request)
      end

      TCellAgent::Instrumentation.safe_block('Handling Dataexposure (request headers)') do
        _handle_dataexpsure_headers(request)
      end

      TCellAgent::Instrumentation.safe_block('Handling Dataexposure (request cookies)') do
        _handler_dataexposure_cookies(request)
      end
    end

    def self.loop_params_hash(method, param_hash, &block)
      param_hash.each do |param_name, param_value|
        if param_value && param_value.is_a?(Hash)
          loop_params_hash(method, param_value, &block)
        elsif !param_value || !param_value.instance_of?(String) || param_value == ''
          next
        else
          block.call(method, param_name, param_value)
        end
      end
    end

    def self.for_params(request, &block)
      get_params = request.GET
      loop_params_hash('get', get_params, &block) if get_params
      post_params = request.POST
      loop_params_hash('post', post_params, &block) if post_params
    end

    def self._handle_dataexpsure_forms(request)
      dataex_policy = TCellAgent.policy(TCellAgent::PolicyTypes::DATALOSS)
      tcell_context = request.env[TCellAgent::Instrumentation::TCELL_ID]
      return unless tcell_context && dataex_policy && dataex_policy.actions_for_form_parameter?

      for_params(request) do |_method, param_name, param_value|
        actions = dataex_policy.get_actions_for_form_parameter(param_name, tcell_context.route_id)
        if actions
          actions.each do |action|
            tcell_context.add_filter_for_request_parameter(param_value, action, param_name)
          end
        end
      end
    end

    def self._handle_dataexpsure_headers(request)
      dataex_policy = TCellAgent.policy(TCellAgent::PolicyTypes::DATALOSS)
      tcell_context = request.env[TCellAgent::Instrumentation::TCELL_ID]
      return unless tcell_context && dataex_policy && dataex_policy.actions_for_headers?

      headers = request.env.select { |k, _v| k.start_with? 'HTTP_' }
      headers.each do |header_name, header_value|
        header_name = header_name.sub(/^HTTP_/, '').tr('_', '-')
        actions = dataex_policy.get_actions_for_header(header_name)
        next unless actions

        actions.each do |action|
          tcell_context.add_filter_for_header_value(header_value, action, header_name)
        end
      end
    end

    def self._handler_dataexposure_cookies(request)
      dataex_policy = TCellAgent.policy(TCellAgent::PolicyTypes::DATALOSS)
      tcell_context = request.env[TCellAgent::Instrumentation::TCELL_ID]
      return unless tcell_context && dataex_policy && dataex_policy.actions_for_cookie?

      request.cookies.each do |cookie_name, cookie_value|
        actions = dataex_policy.get_actions_for_cookie(cookie_name)
        next unless actions

        actions.each do |action|
          tcell_context.add_filter_for_cookie_value(cookie_value, action, cookie_name)
        end
      end
    end
  end
end
