# See the file "LICENSE" for the full license governing this code.

require 'rails'
require 'uri'
require 'tcell_agent/agent'
require 'tcell_agent/sensor_events/sensor'
require 'tcell_agent/sensor_events/server_agent'
require 'tcell_agent/sensor_events/util/sanitizer_utilities'

require 'tcell_agent/configuration'
require 'tcell_agent/instrumentation'
require 'tcell_agent/rails/responses'

module TCellAgent
  module Instrumentation
    module Rails
      module Middleware
        class BodyFilterMiddleware
          def initialize(app)
            @app = app
          end

          def call(env)
            start_time = (Time.now.to_f * 1000).to_i

            response = @app.call(env)

            if TCellAgent.configuration.should_intercept_requests?
              TCellAgent::Instrumentation.safe_block('Handling reporting metrics') do
                response_time = (Time.now.to_f * 1000).to_i - start_time
                TCellAgent.report_metrics(response_time, env[TCellAgent::Instrumentation::TCELL_ID])
              end
            end

            response
          end
        end
      end
    end
  end
end
