# See the file "LICENSE" for the full license governing this code.

require 'rails'
require 'uri'
require 'tcell_agent/agent'
require 'tcell_agent/sensor_events/sensor'
require 'tcell_agent/sensor_events/server_agent'
require 'tcell_agent/sensor_events/util/sanitizer_utilities'

require 'tcell_agent/rails/auth/userinfo'
require 'cgi'

require 'tcell_agent/instrumentation'

module TCellAgent
  module Instrumentation
    module Rails
      module Middleware
        class GlobalMiddleware
          def initialize(app)
            @app = app
          end

          def call(env)
            if TCellAgent.configuration.should_intercept_requests?
              request = Rack::Request.new(env)

              TCellAgent::Instrumentation.safe_block('Setting session_id & user_id') do
                if request.session
                  env[TCellAgent::Instrumentation::TCELL_ID].session_id =
                    request.session['session_id']
                  env[TCellAgent::Instrumentation::TCELL_ID].user_id =
                    TCellAgent::UserInformation.get_user_from_request(request)
                end
              end

              TCellAgent::Instrumentation.safe_block('Setting hmac_session_id') do
                if request.env[TCellAgent::Instrumentation::TCELL_ID].session_id
                  env[TCellAgent::Instrumentation::TCELL_ID].hmac_session_id =
                    TCellAgent::SensorEvents::Util.hmac(request.env[TCellAgent::Instrumentation::TCELL_ID].session_id)
                end
              end
            end

            @app.call(env)
          end
        end
      end
    end
  end
end
