require 'rails'
require 'tcell_agent'
require 'tcell_agent/sensor_events/app_config_setting_event'
require 'tcell_agent/sensor_events/server_agent'

module TCellAgent
  module Instrumentation
    module Rails
      def self.framework_details
        { 'app_framework' => 'Rails',
          'app_framework_version' => ::Rails.version }
      end

      def self.send_settings
        TCellAgent::Instrumentation.safe_block('Reporting Rails settings') do
          rails_config = ::Rails.application.config

          # Defaults to true
          csrf_protection = rails_config.action_controller.allow_forgery_protection || true
          TCellAgent.send_event(
            TCellAgent::SensorEvents::AppConfigSettingEvent.new(
              'Rails', 'core', '', 'csrf_protection', csrf_protection
            )
          )

          # Defaults to false if nil
          mass_assignment_allowed = rails_config.action_controller.permit_all_parameters || false
          TCellAgent.send_event(
            TCellAgent::SensorEvents::AppConfigSettingEvent.new(
              'Rails', 'core', '', 'mass_assignment_allowed', mass_assignment_allowed
            )
          )

          # Defaults to never
          session_expire = rails_config.session_options[:expire_after] || -1
          TCellAgent.send_event(
            TCellAgent::SensorEvents::AppConfigSettingEvent.new(
              'Rails', 'session', '', 'timeout', session_expire
            )
          )
        end
      end
    end
  end
end
