module TCellAgent
  module Rust
    class NativeAgentResponse
      attr_reader :errors, :response

      def initialize(native_method_name, response, response_len)
        @response = {}
        @errors = []

        if response_len < 0
          @errors.push(
            "Error response from `#{native_method_name}` in native library method: #{response_len}"
          )
          return
        end

        begin
          @response = JSON.parse(response.get_string(0, response_len))
          if @response['error']
            @errors.push(
              "#{native_method_name} returned an error: #{@response['error']}"
            )
            @response = {}
          end
          if @response['errors']
            @response['errors'].each do |error|
              @errors.push(
                "#{native_method_name} returned an error: #{error}"
              )
              @response = {}
            end
          end
        rescue JSON::ParserError
          @errors.push(
            "Could not parse json response from `#{native_method_name}` in native library."
          )
          @response = {}
        end
      end
    end
  end
end
