require 'tcell_agent/rust/models'

module TCellAgent
  module Rust
    module NativeLibrary
      require 'ffi'
      extend FFI::Library

      extension = '.so'
      variant = ''
      if /cygwin|mswin|mingw|bccwin|wince|emx/ =~ RUBY_PLATFORM
        variant = '-x64'
        extension = '.dll'
      elsif /darwin/ =~ RUBY_PLATFORM
        extension = '.dylib'
      elsif /musl/ =~ RUBY_PLATFORM
        variant = '-alpine'
      end

      begin
        ffi_lib File.join(File.dirname(__FILE__),
                          "libtcellagent#{variant}#{extension}")

        # All the rust library calls have the following response api:
        #
        # result [int]: 0+ length of buffer_out answer
        #               -1 general error
        #               -2 buffer_out is not big enough for response
        #               -3 buffer_out is null

        attach_function :create_agent, %i[pointer size_t pointer size_t], :int
        attach_function :free_agent, [:pointer], :int
        attach_function :request_policies, %i[pointer pointer size_t], :int
        attach_function :poll_new_policies, %i[pointer pointer size_t], :int
        attach_function :appfirewall_apply, %i[pointer pointer size_t pointer size_t], :int
        attach_function :patches_apply, %i[pointer pointer size_t pointer size_t], :int
        attach_function :suspicious_quick_check_apply, %i[pointer pointer size_t], :int
        attach_function :cmdi_apply, %i[pointer pointer size_t pointer size_t], :int
        attach_function :get_headers, %i[pointer pointer size_t pointer size_t], :int
        attach_function :get_js_agent_script_tag, %i[pointer pointer size_t pointer size_t], :int
        attach_function :check_http_redirect, %i[pointer pointer size_t pointer size_t], :int
        attach_function :report_metrics, %i[pointer pointer size_t pointer size_t], :int
        attach_function :login_fraud_apply, %i[pointer pointer size_t pointer size_t], :int
        attach_function :file_access_apply, %i[pointer pointer size_t pointer size_t], :int

        attach_function :send_sanitized_events, %i[pointer pointer size_t pointer size_t], :int
        attach_function :log_message, %i[pointer pointer size_t pointer size_t], :int

        attach_function :update_policies, %i[pointer pointer size_t pointer size_t], :int
        attach_function :test_agent, %i[pointer size_t pointer size_t], :int

        def self.common_lib_available?
          true
        end
      rescue LoadError => e
        logger = TCellAgent::ModuleLogger.new(TCellAgent::RubyLogger.new, name)
        logger.error("Failed loading agent library. #{e.message}")
        logger.exception(e)

        def self.common_lib_available? # rubocop:disable Lint/DuplicateMethods
          false
        end
      end
    end
  end
end
