# See the file "LICENSE" for the full license governing this code.

require 'tcell_agent/sensor_events/util/sanitizer_utilities'
require 'tcell_agent/sensor_events/sensor'

module TCellAgent
  module SensorEvents
    class DlpEvent < TCellSensorEvent
      FOUND_IN_BODY = 'body'.freeze
      FOUND_IN_LOG = 'log'.freeze
      FOUND_IN_CONSOLE = 'console'.freeze

      FRAMEWORK_VARIABLE_SESSION_ID = 'session_id'.freeze

      REQUEST_CONTEXT_FORM = 'form'.freeze
      REQUEST_CONTEXT_COOKIE = 'cookie'.freeze
      REQUEST_CONTEXT_HEADER = 'header'.freeze

      def initialize(route_id, raw_uri, found_in, id = nil, hmac_session_id = nil, user_id = nil)
        super('dlp')
        self['rid'] = route_id if route_id
        self['found_in'] = found_in
        self['uri'] = Util.strip_uri_values(raw_uri) if raw_uri
        self['sid'] = hmac_session_id if hmac_session_id
        self['uid'] = user_id if user_id
        self['rule'] = id if id
      end

      def for_database(database, schema, table, field)
        self['type'] = 'db'
        self['db'] = database
        self['schema'] = schema
        self['table'] = table
        self['field'] = field
        self
      end

      def for_framework(variable)
        self['type'] = 'fw'
        self['context'] = 'framework'
        self['variable'] = variable
        self
      end

      def for_request(variable_context, variable)
        self['type'] = 'req'
        self['context'] = variable_context
        self['variable'] = variable
        self
      end
    end
  end
end
