# See the file "LICENSE" for the full license governing this code.

require 'tcell_agent/sensor_events/util/sanitizer_utilities'
require 'tcell_agent/sensor_events/sensor'
require 'tcell_agent/sensor_events/util/utils'
require 'etc'

module TCellAgent
  module SensorEvents
    class ServerAgentPackagesSensorEvent < TCellSensorEvent
      include TCellAgent::ModuleLoggerAccess

      def initialize
        super('server_agent_packages')
        @flush = true
        @ensure = true
        packages = []
        Gem.loaded_specs.values.map do |x|
          begin
            if x.name
              package = { 'n' => x.name, 'v' => x.version.version }
              packages.push(package)
              module_logger.debug("Adding packages #{x.name}")
            end
          rescue StandardError => e
            module_logger.error("Exception adding package: #{e.message}")
            module_logger.exception(e)
          end
        end
        self['packages'] = packages
      end
    end

    class AppRoutesSensorEvent < TCellSensorEvent
      def initialize(uri, method, route_id, params = nil, destination = nil)
        super('appserver_routes')
        @flush = false
        @ensure = true
        method = method.downcase
        self['uri'] = uri
        self['method'] = method
        self['rid'] = route_id
        self['params'] = params if params
        self['destination'] = destination if destination
      end
    end
  end
end
