# See the file "LICENSE" for the full license governing this code.

require 'cgi'
require 'uri'
require 'openssl'

module TCellAgent
  module SensorEvents
    module Util
      def self.hmac(data)
        hmac_key = Util.get_hmac_key

        h = OpenSSL::HMAC.hexdigest(OpenSSL::Digest.new('sha256'), hmac_key.to_s, data)

        h[0...h.length / 2]
      end

      def self.strip_values_query_string(query)
        params = CGI.parse(query)
        params.each do |param_name, param_values|
          next if param_values.nil? || param_values.empty?

          params[param_name] = ['']
        end
        params.map { |k, v| "#{k}=#{v.join(',')}" }.join('&')
      end

      def self.strip_uri_values(uri_string)
        uri = URI(uri_string)
        query = uri.query
        uri.query = strip_values_query_string(query) if query
        uri.to_s
      end

      def self.get_hmac_key
        return TCellAgent.configuration.hmac_key if TCellAgent.configuration.hmac_key
        return TCellAgent.configuration.app_id if TCellAgent.configuration.app_id

        'tcell_hmac_key'
      end
    end
  end
end
