if defined?(Puma.cli_config)
  if Puma.cli_config.options[:preload_app]
    if Puma.cli_config.options[:workers] == 0
      # Puma is running in single mode, so run both the initial instrumentation and
      # start the agent
      Puma::Runner.class_eval do
        alias_method :tcell_original_start_server, :start_server
        def start_server
          TCellAgent.thread_agent.start('Puma')

          tcell_original_start_server
        end
      end

    else
      Puma::Server.class_eval do
        alias_method :tcell_original_run, :run

        if defined?(Gem::Version) &&
           defined?(Puma::Const::PUMA_VERSION) &&
           (Gem::Version.new(Puma::Const::PUMA_VERSION) < Gem::Version.new('5.1.0'))
          def run(background = true)
            TCellAgent.thread_agent.start('Puma')
            original_run(background, options)
          end
        else
          def run(background = true, thread_name: 'server')
            TCellAgent.thread_agent.start('Puma')
            original_run(background, :thread_name => thread_name)
          end
        end
      end
    end
  else
    # this ensures instrumentation runs for preload_app = false.
    # Instrumentation will run for each worker but there's
    # nothing we can do about that (Unicorn's preload_app behaves the same way)
    Puma::Server.class_eval do
      alias_method :tcell_original_run, :run

      if defined?(Gem::Version) &&
         defined?(Puma::Const::PUMA_VERSION) &&
         (Gem::Version.new(Puma::Const::PUMA_VERSION) < Gem::Version.new('5.1.0'))
        def run(background = true)
          TCellAgent.thread_agent.start('Puma')
          tcell_original_run(background)
        end
      else
        def run(background = true, thread_name: 'server')
          TCellAgent.thread_agent.start('Puma')
          original_run(background, :thread_name => thread_name)
        end
      end
    end
  end
end
