Unicorn::HttpServer.class_eval do
  # - This will be false when preload_app is false (even when unicorn is sent USR2 SIGNAL)
  # - This check also ensures that a server is running as opposed to a different command such
  # as `bundle exec rails runner User.count`.
  unless Unicorn::HttpServer::START_CTX && Unicorn::HttpServer::START_CTX[0]
    require 'tcell_agent/rails/railties/tcell_agent_unicorn_railties'
  end

  # This only gets instrumented when preload_app is true
  # this only runs when preload_app=true because when preload_app=false
  # the gems aren't loaded early enough for tcell to override
  # the class definitions
  alias_method :tcell_init_worker_process, :init_worker_process
  def init_worker_process(work)
    start_process = tcell_init_worker_process(work)

    TCellAgent.logger.debug('Instrumenting Unicorn server', 'TCellAgent::Unicorn')

    TCellAgent.thread_agent.start('Unicorn Worker')

    start_process
  end

  # This gets called when unicorn receives the HUP signal to reload its config.
  # Tcell also needs to ensure its config is reloaded and services are started
  # or stopped accordingly
  alias_method :tcell_load_config!, :load_config!
  def load_config!
    result_if_exists = tcell_load_config!

    hot_reload_tcell_agent

    result_if_exists
  end

  def hot_reload_tcell_agent
    TCellAgent::Instrumentation.safe_block('Reloading Tcell Config') do
      TCellAgent.logger.debug('[TCellAgent] Reloading configuration', 'TCellAgent::Unicorn')

      new_config = TCellAgent::Configuration.new

      TCellAgent.configuration = new_config
    end
  end
end
