require 'tcell_agent/agent'
require 'tcell_agent/configuration'
require 'tcell_agent/sensor_events/agent_setting_event'
require 'tcell_agent/sensor_events/server_agent'
require 'thread'

module TCellAgent
  def self.report_settings
    Thread.new do
      TCellAgent::Instrumentation.safe_block('Instrumenting Server Packages') do
        event = TCellAgent::SensorEvents::ServerAgentPackagesSensorEvent.new
        TCellAgent.send_event(event)
      end

      TCellAgent::Instrumentation.safe_block('Instrumenting Native Lib Status') do
        require 'tcell_agent/rust/native_agent'

        TCellAgent.send_event(
          TCellAgent::SensorEvents::AgentSettingEvent.new(
            'native_lib_loaded',
            TCellAgent::Rust::NativeLibrary.common_lib_available?.to_s
          )
        )
      end

      if defined?(::Rails)
        TCellAgent::Instrumentation.safe_block('Instrumenting routes') do
          TCellAgent::Instrumentation::Rails.instrument_routes
        end
      end
    end
  end
end
