# See the file "LICENSE" for the full license governing this code.

require 'sinatra/base'
require 'sinatra'
require 'tcell_agent/agent'
require 'tcell_agent/instrumentation'

module TCellAgent
  class Sinatra::Response # rubocop:disable Style/ClassAndModuleChildren
    include Sinatra

    alias_method :original_finish, :finish
    def finish
      status, headers, response = original_finish

      TCellAgent::Instrumentation.safe_block('Setting Headers') do
        headers_policy = TCellAgent.policy(TCellAgent::PolicyTypes::HEADERS)
        policy_headers = headers_policy.get_headers(
          headers['Content-Type'],
          request.env[TCellAgent::Instrumentation::TCELL_ID]
        )
        policy_headers.each do |header_info|
          header_name = header_info['name']
          header_value = header_info['value']
          existing_header_value = headers[header_name]
          headers[header_name] = if existing_header_value
                                   "#{existing_header_value}, #{header_value}"
                                 else
                                   header_value
                                 end
        end
        response = [status, headers, active_response]
      end

      [status, headers, response]
    end
  end
end
