module TCellAgent
  module Utils
    module Strings
      BLANK_RE = /\A[[:space:]]*\z/.freeze

      def self.blank?(str)
        str.nil? || str.empty? || BLANK_RE === str
      end

      def self.present?(str)
        !blank?(str)
      end

      def self.remove_trailing_slash(path)
        return path.chomp('/') if path && path != '/'

        path
      end

      # emulate the java String.hashcode() without upcasting to BigInt
      def self.java_hashcode(str)
        result = 0
        str.each_codepoint do |cp|
          # prevent overflow into BigInt which would cause heap allocs + emulate c-style int32 signed add overflow
          result = ((((((result & 0x07FFFFFF) << 5) - result) + cp) + 0x80000000) & 0xFFFFFFFF) - 0x80000000
        end
        result
      end
    end
  end
end
